function [Best, kopts, eigvals, B0Opt, B1Opt] = Func_RSC(X, Y, lambdas)
% Criterion: || Y - X B ||_F^2/2 + lambda^2 * rank(B)/2
% The theoretical choice for lambda (to suppress the noise term) is
% 2*sigma^2 * (size(Y,2) + rank(X))
% Output: 
%   Best = B0Opt * B1Opt is the final estimate. B0Opt can be used for
%           constructing new factors. 
%   eigvals: eigenvalues of the target matrix defined in the RSC paper.

tmpUniEst = X' * Y;
tmpSigma = X' * X;

tmpSigmaGInv = pinv(tmpSigma); % generalized inverse
B_unpen = tmpSigmaGInv * tmpUniEst; % unpenalized (LS) estimate
% Perform spectral decomposition
eigTarget = tmpUniEst' * B_unpen;
[eigV0, eigD0] = eig((eigTarget+eigTarget')/2);
[eigDdiag, IX] = sort(diag(eigD0), 'descend');
eigD = eigD0(IX, IX); 
eigV = eigV0(:, IX);
eigvals = eigDdiag;

[m, n] = size(Y); d = size(X, 2);
r = min(n, sum(svd(X)>1e-6)); % r = min(n, d); 
Bs = zeros(d, n, r);
B0 = B_unpen * eigV;
B1 = eigV';
% The above precomputation is the same for all lambda vaues


if numel(lambdas) == 1
    lambda = lambdas;

    % Choose the rank
    k = sum(eigvals> lambda^2); % the extra step in comparison to RRR
    % Obtain the estimate
    Best = B0(:,1:k) * B1(1:k, :);
    kopts = k;
    B0Opt = B0(:,1:k);
    B1Opt = B1(1:k, :); 
else
    Best = zeros(d, n, numel(lambdas));
    kopts = -ones(1, numel(lambdas));
    for lambdaInd=1:numel(lambdas)
        lambda = lambdas(lambdaInd);
        k = sum(eigvals> lambda^2); % the extra step in comparison to RRR
        kopts(lambdaInd) = k;
        Best(:, :, lambdaInd) = B0(:,1:k) * B1(1:k, :);
    end

    B0Opt = B0; 
    B1Opt = B1;   
end


